<?php

namespace App\domain\travelport;

use App\Models\TpCombinedOffer;
use App\Models\TpFlight;
use App\Models\TpProduct;
use Carbon\Carbon;
use Carbon\CarbonInterval;
use Exception;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;

class QueryOffersPageAction
{
    protected Collection $productOffersCollection;
    protected string $catalogId;

    public function execute(string $catalogId, int $page): array
    {
        $this->catalogId = $catalogId;
        $this->productOffersCollection = $this->getProductOffersCollection($catalogId, $page);

        $this->attachReferenceData();

        return $this->productOffersCollection
            ->groupBy('combinabilityCode')
            ->toArray();
    }

    protected function getProductOffersCollection(string $catalogId, int $page): Collection
    {
        $records = TpCombinedOffer::query()
            ->where('catalogIdentifier', $catalogId)
            ->orderBy('bestCombinablePrice')
            ->skip(($page - 1) * 20)
            ->take(20)
            ->get();

        return $records
            ->pluck('offers')
            ->flatMap(function ($offers) {
                return json_decode($offers);
            });
    }

    protected function attachReferenceData(): void
    {
        $productAirList = TpProduct::query()->where('catalogIdentifier', $this->catalogId)->get();

        $this->productOffersCollection = $this->productOffersCollection
            ->map(function ($product) use ($productAirList) {
                $productAir = $productAirList->where('productId', $product->id)->first();
                $flightSegments = new Collection(json_decode($productAir['flightSegment']));
                $productAir['FlightSegment'] = $flightSegments;
                $productAir['PassengerFlight'] = new Collection(json_decode($productAir['passengerFlight']));;
                $flightSegments = $this->addFlightDetailsToSegments($product->authority, $flightSegments);
                $segmentDetails = $this->extractFlightSegmentDetails($flightSegments);
                $durations = $this->calculateProductDurations($productAir->toArray());

                return [
                    ...(array)$product,
                    'productAir' => $productAir,
                    'fromAirportCarrierCode' => $segmentDetails['fromAirportCarrierCode'],
                    'fromAirportAircraftCode' => $segmentDetails['fromAirportAircraftCode'],
                    'toAirportCarrierCode' => $segmentDetails['toAirportCarrierCode'],
                    'toAirportAircraftCode' => $segmentDetails['toAirportAircraftCode'],
                    'originDepartureTime' => $segmentDetails['originDepartureTime'],
                    'destinationArrivalTime' => $segmentDetails['destinationArrivalTime'],
                    'duration' => $durations['Duration'],
                    'numberOfStops' => $flightSegments->count() - 1,
                    'segments' => $flightSegments->toArray(),
                    'durationInHours' => $durations['Hours'],
                    'numberOfBookableSeats' => $productIAir['Quantity'] ?? 0,
                ];
            });
    }

    protected function addFlightDetailsToSegments(string $authority, Collection $flightSegments): Collection
    {
        $flightList = TpFlight::query()->where('catalogIdentifier', $this->catalogId)->get();

        return $flightSegments->map(function ($segment) use ($flightList) {
            $flightSegment = get_object_vars($segment);
            $flightDetails = $flightList
                ->where('flightId', $flightSegment['Flight']->FlightRef)
                ->first()
                ->toArray();

            $flightSegment['FlightDetails'] = [
                ...$flightDetails,
                'Arrival' => (array)json_decode($flightDetails['arrival']),
                'Departure' => (array)json_decode($flightDetails['departure'])
            ];

            return $flightSegment;
        });
    }

    protected function extractFlightSegmentDetails(Collection $segments): array
    {
        $firstFlight = Collection::wrap($segments->first()["FlightDetails"]);
        $lastFlight = Collection::wrap($segments->last()["FlightDetails"]);

        return [
            'originDepartureTime' => $this->getDepartureOrArrivalTime($firstFlight, direction: "Departure"),
            'destinationArrivalTime' => $this->getDepartureOrArrivalTime($lastFlight, direction: "Arrival"),
            'fromAirportCarrierCode' => $firstFlight['carrier'],
            'toAirportCarrierCode' => $lastFlight['carrier'],
            'fromAirportAircraftCode' => $firstFlight['equipment'],
            'toAirportAircraftCode' => $lastFlight['equipment'],
            'fromAirline' => isset($lastFlight['operatingCarrierName'])
                ? $lastFlight['operatingCarrierName']
                : "",
        ];
    }

    protected function calculateDirectionDuration(Collection $segments): array
    {
        try {
            $durations = $segments->pluck('FlightDetails.duration');

            $totalHours = 0;
            $totalMinutes = 0;

            $durations->each(function ($durationString) use (&$totalHours, &$totalMinutes) {
                $duration = CarbonInterval::fromString($durationString);
                $totalHours += $duration->hours;
                $totalMinutes += $duration->minutes;
            });

            $totalHours += floor($totalMinutes / 60);
            $totalMinutes = $totalMinutes % 60;

            $durationHours = $totalHours + ($totalMinutes ? $totalMinutes / 60 : 0);

            return ["Duration" => "PT{$totalHours}H{$totalMinutes}M", "Hours" => $durationHours];

        } catch (Exception $e) {
            return ["Duration" => "", "Hours" => ""];
        }
    }

    private function getDepartureOrArrivalTime(Collection $firstFlight, string $direction): string
    {
        $departureDate = $firstFlight->get($direction)['date'];
        $departureTime = $firstFlight->get($direction)['time'];

        return Carbon::createFromFormat('Y-m-d H:i:s', $departureDate . ' ' . $departureTime)->toISOString();
    }

    protected function calculateProductDurations(array $productAir): array
    {
        try {
            $totalHours = 0;
            $totalMinutes = 0;

            $totalDuration = CarbonInterval::fromString($productAir["totalDuration"]);

            $totalHours += $totalDuration->hours;
            $totalMinutes += $totalDuration->minutes;

            $totalHours += floor($totalMinutes / 60);
            $totalMinutes = $totalMinutes % 60;

            $durationHours = $totalHours + ($totalMinutes ? $totalMinutes / 60 : 0);

            return ["Duration" => $productAir["totalDuration"], "Hours" => $durationHours];

        } catch (Exception $e) {
            Log::error($e->getMessage());
            return ["Duration" => "", "Hours" => ""];
        }
    }
}
